package net.pricefx.formulaengine.scripting.inputbuilder;

import com.fasterxml.jackson.core.JsonProcessingException;
import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.common.apibuilder.clicoll.CalculableLineItemCollectionBuilder;
import net.pricefx.lmformulaengine.Util;
import net.pricefx.server.dto.calculation.ConfiguratorEntry;
import net.pricefx.server.dto.calculation.ContextParameter;

import java.util.List;
import java.util.Map;

public abstract class AbstractInputBuilder<T extends AbstractInputBuilder, VT> {
    protected ContextParameter cp;
    protected final InputBuilderContext builderContext;

    public AbstractInputBuilder(InputBuilderContext builderContext, InputType type, String paramName) {
        this.builderContext = builderContext;
        cp = new ContextParameter(type, paramName,builderContext.getEngineContext().getTargetDate(), null);
    }

    public AbstractInputBuilder(InputBuilderContext builderContext) {
        this.builderContext = builderContext;
    }

    protected Object _getInput() {
        return builderContext.getEngineContext().getParameter(cp.getName());
    }

    public final Object getInput() {
        if(builderContext.getEngineContext().getParameter(cp.getName()) == null)
            builderContext.getEngineContext().addMissingParameter(cp);  //this will effectively prevent the addMissingParameter call in the original functions; hence make sure our amended CP is added

        return _getInput();
    }

    public T setLabel(String label) {
        cp.setLabel(label);
        return (T) this;
    }

    public T setLabelTranslations(Map<String,String> translations) throws JsonProcessingException {
        cp.setLabelTranslationsFromMap(translations);
        return (T) this;
    }

    public T setRequired(Boolean required) {
        cp.setRequired(required);
        return (T) this;
    }

    public T setReadOnly(Boolean ro) {
        cp.setReadOnly(ro);
        return (T) this;
    }

    public T setAlwaysEditable(Boolean ae) {
        cp.setAlwaysEditable(ae);
        return (T) this;
    }

    public T setValue(VT value) {
        cp.setValue(value);
        return (T) this;
    }

    public T setValueHint(String valueHint) {
        cp.setValue(valueHint);
        return (T) this;
    }

    public T setNoRefresh(boolean noRefresh) {
        cp.setConfigParameter("noRefresh", noRefresh);
        return (T) this;
    }

    public T setConfigValues(List<String> configValues) throws JsonProcessingException {
        cp.addParameterConfigEntry(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_VALUES, Util.getObjectMapper().writeValueAsString(configValues));
        return (T) this;
    }

    public T setParameterGroup(String pg) {
        cp.setParameterGroup(pg);
        return (T) this;
    }

    public T setPlaceholderText(String text) {
        cp.setFormattingOption("placeholder",text);
        return (T) this;
    }

    public T setImportable(boolean importable) {
        cp.setConfigParameter("importable",importable);
        return (T) this;
    }

    public T setExportable(boolean exportable) {
        cp.setConfigParameter("exportable",exportable);
        return (T) this;
    }

    public T setTheme(String theme) {
        cp.setFormattingOption("theme",theme);
        return (T) this;
    }

    public T setAppearance(String appearance) {
        cp.setFormattingOption("appearance",appearance);
        return (T) this;
    }

    public T setSize(String size) {
        cp.setFormattingOption("size",size);
        return (T) this;
    }

    public T setTitle(String title) {
        cp.setFormattingOption("title",title);
        return (T) this;
    }

    public T setCaption(String caption) {
        cp.setFormattingOption("caption",caption);
        return (T) this;
    }

    public T setHelpText(String helpText) {
        cp.setFormattingOption("helpText",helpText);
        return (T) this;
    }

    public T setHelpLink(String helpLink) {
        cp.setFormattingOption("helpLink",helpLink);
        return (T) this;
    }

    public T setIcon(String icon) {
        cp.setFormattingOption("icon",icon);
        return (T) this;
    }

    public T setIconPosition(String iconPosition) {
        cp.setFormattingOption("iconPosition",iconPosition);
        return (T) this;
    }

    public T setLabelPlacement(String labelPlacement) {
        cp.setFormattingOption("labelPlacement",labelPlacement);
        return (T) this;
    }

    public T setLayoutGridName(String layoutGridName) {
        cp.setFormattingOption("layoutGridName",layoutGridName);
        return (T) this;
    }

    public T setErrorMessage(String msg) {
        cp.setConfigParameter("errorMessage",msg);
        return (T) this;
    }

    public T setWarningMessage(String msg) {
        cp.setConfigParameter("warningMessage",msg);
        return (T) this;
    }

    public T setSuccessMessage(String msg) {
        cp.setConfigParameter("successMessage",msg);
        return (T) this;
    }

    public T setUserGroupView(String ugv) {
        cp.setConfigParameter("userGroupView",ugv);
        return (T) this;
    }

    public T setUserGroupEdit(String uge) {
        cp.setConfigParameter("userGroupEdit",uge);
        return (T) this;
    }

    public T setDisplayMode(String displayMode) {
        cp.setFormattingOption("displayMode",displayMode);
        return (T) this;
    }

    public T setAccessCode(String accessCode) {
        cp.setFormattingOption("accessCode",accessCode);
        return (T) this;
    }

    public T setAlign(String align) {
        cp.setFormattingOption("align",align);
        return (T) this;
    }

    public T setLabelStyle(String labelStyle) {
        cp.setFormattingOption("labelStyle",labelStyle);
        return (T) this;
    }

    public T setAutoComplete(boolean autoComplete) {
        cp.setConfigParameter("autoComplete",autoComplete);
        return (T) this;
    }

    public T setAutoFocus(boolean autoFocus) {
        cp.setConfigParameter("autoFocus",autoFocus);
        return (T) this;
    }

    public T setDisabled(boolean disabled) {
        cp.setConfigParameter("disabled",disabled);
        return (T) this;
    }

    public T setCustomAttributeName(String name) {
        cp.setFormattingOption("customAttributeName",name);
        return (T) this;
    }

    public T setCustomAttributeValue(String value) {
        cp.setFormattingOption("customAttributeValue",value);
        return (T) this;
    }

    public Map<String,Object> buildMap() {
        return Util.getObjectMapper().convertValue(cp, Map.class);
    }

    public ContextParameter buildContextParameter() {
        return cp;
    }

    public T addOrUpdateInput(CalculableLineItemCollectionBuilder clicBuilder, String lineId) {
        if(clicBuilder != null)
            clicBuilder.addOrUpdateInput(lineId,buildMap());
        return (T) this;
    }

    public T addToConfiguratorEntry(ConfiguratorEntry ce) {
        ce.addParameter(cp);
        return (T) this;
    }
}
