package net.pricefx.logic.core.metadata;

import net.pricefx.logic.core.PfxFilters;
import net.pricefx.logic.core.TypeCode;
import net.pricefx.logic.core.config.Partition;
import net.pricefx.restapiclient.PfxClientException;
import net.pricefx.restapiclient.api.ConfigurationApi;
import net.pricefx.restapiclient.api.DatamartApi;
import net.pricefx.restapiclient.api.GeneralApi;
import net.pricefx.restapiclient.criteria.Criterion;
import net.pricefx.restapiclient.model.BaseMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import static net.pricefx.common.api.PriceFxInterface.FIELD_DM_SOURCENAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_TYPEDID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_UNIQUENAME;
import static net.pricefx.common.api.PriceFxInterface.PARAM_FILTER_CRITERIA_FIELDNAME;

public class MetadataCache {
    private static final Logger log = LoggerFactory.getLogger(MetadataCache.class);

    protected Partition partition;

    protected Map<TypeCode, Map<String, BaseMap>> meta = new EnumMap(TypeCode.class);
    protected Map<TypeCode, Map<Object, Map<String, BaseMap>>> metaInstances = new EnumMap(TypeCode.class);
    protected Map<TypeCode, Map<String, BaseMap>> objectExtensions = new EnumMap(TypeCode.class);
    protected Map<TypeCode, List<BaseMap>> metasDatamart = new EnumMap(TypeCode.class);
    protected Map<TypeCode, Map<String, BaseMap>> dataObjects = new EnumMap(TypeCode.class);

    public MetadataCache(Partition partition) {
        this.partition = partition;
        clear();
    }

    public void clear() {
        meta.clear();
        metaInstances.clear();
        dataObjects.clear();
        objectExtensions.clear();
        metasDatamart.clear();
    }

    private Map<String, BaseMap> groupBySorted(String groupByFieldName, List<BaseMap> metas) {
        Map<String, BaseMap> result = new TreeMap();
        for (BaseMap metadata : metas) {
            result.put(metadata.getStringValue(groupByFieldName), metadata);
        }
        return result;
    }

    private Map<Object, Map<String, BaseMap>> groupByInstanceSorted(String groupByFieldName, List<BaseMap> metas) {
        Map<Object, Map<String, BaseMap>> pxMetaMap = new TreeMap();

        for (BaseMap metadata : metas) {
            Object groupFieldValue = metadata.get(groupByFieldName);
            if (groupFieldValue instanceof Number) {
                groupFieldValue = ((Number) groupFieldValue).longValue();
            }
            if (!pxMetaMap.containsKey(groupFieldValue)) {
                pxMetaMap.put(groupFieldValue, new TreeMap());
            }
            pxMetaMap.get(groupFieldValue).put(metadata.getStringValue(PARAM_FILTER_CRITERIA_FIELDNAME), metadata);
        }

        return pxMetaMap;
    }


    public Map<String, BaseMap> getMetas(TypeCode typeCode) throws PfxClientException {
        if (meta.get(typeCode) == null) {
            try {
                TypeCode typeCodeMeta = typeCode.getMetaTypeCode();
                List<BaseMap> metas = GeneralApi.fetch(partition, typeCodeMeta.getShortName());
                meta.put(typeCode, groupBySorted(PARAM_FILTER_CRITERIA_FIELDNAME, metas));
            } catch (Exception e) {
                meta.put(typeCode, new HashMap<>());
                throw e;
            }
        }

        return meta.get(typeCode);
    }

    public Map<Object, Map<String, BaseMap>> getMetasForInstances(TypeCode typeCode) throws PfxClientException {
        if (metaInstances.get(typeCode) == null) {
            try {
                TypeCode typeCodeMeta = typeCode.getMetaTypeCode();
                String[] groupByFieldName = typeCode.getIdentifierFieldNames();
                List<BaseMap> metas = GeneralApi.fetch(partition, typeCodeMeta.getShortName());
                metaInstances.put(typeCode, groupByInstanceSorted(groupByFieldName[0], metas));
            } catch (Exception e) {
                metaInstances.put(typeCode, new HashMap<>());
                throw e;
            }
        }

        return metaInstances.get(typeCode);
    }


    public Map<String, BaseMap> getDataObjects(TypeCode typeCode) {
        if (dataObjects.get(typeCode) == null) {
            try {
                List<BaseMap> metas = GeneralApi.fetch(partition, typeCode.getShortName());
                dataObjects.put(typeCode, groupBySorted(FIELD_UNIQUENAME, metas));
            } catch (Exception e) {
                dataObjects.put(typeCode, new HashMap<>());
            }
        }

        return dataObjects.get(typeCode);
    }

    public Map<String, BaseMap> getObjectExtensions(TypeCode typeCode) throws PfxClientException {
        if (objectExtensions.get(typeCode) == null) {
            try {
                String configurationName = typeCode.name().toLowerCase();
                objectExtensions.put(typeCode, (Map<String, BaseMap>) ConfigurationApi.fetchSettingAsMap(partition, configurationName));
            } catch (PfxClientException e) {
                objectExtensions.put(typeCode, new HashMap<>());
                throw e;
            }
        }

        return objectExtensions.get(typeCode);
    }

    public boolean isMetasDatamartLoaded(TypeCode typeCode) {
        return metasDatamart.get(typeCode) != null;
    }

    public List<BaseMap> getMetasDatamart(TypeCode typeCode) throws PfxClientException {
        if (metasDatamart.get(typeCode) == null) {
            try {
                metasDatamart.put(typeCode, DatamartApi.getFcs(partition, typeCode.getShortName()));
            } catch (Exception e) {
                metasDatamart.put(typeCode, new ArrayList<>());
                throw e;
            }
        }

        return metasDatamart.get(typeCode);
    }

    /**
     *
     * @param typeCode
     * @param uniqueFieldName either FIELD_UNIQUENAME or FIELD_TYPEDID
     * @param datamartName
     * @return
     */
    protected BaseMap getMetasDatamart(TypeCode typeCode, String uniqueFieldName, String datamartName) throws PfxClientException {
        List<BaseMap> datamarts = getMetasDatamart(typeCode);
        for (BaseMap datamart : datamarts) {
            if (datamart.get(uniqueFieldName).equals(datamartName)) {
                return datamart;
            }
        }

        return null;
    }

    public BaseMap getMetasDatamartByName(TypeCode typeCode, String datamartName) throws PfxClientException {
        return getMetasDatamart(typeCode, FIELD_UNIQUENAME, datamartName);
    }

    public BaseMap getMetasDatamartByTypedId(TypeCode typeCode, String datamartName) throws PfxClientException {
        return getMetasDatamart(typeCode, FIELD_TYPEDID, datamartName);
    }

    public BaseMap getMetasDatamartBySourceName(TypeCode typeCode, String datamartName) throws PfxClientException {
        return getMetasDatamart(typeCode, FIELD_DM_SOURCENAME, datamartName);
    }

    public Map<String, BaseMap> getMetasForPriceListOrGrid(TypeCode typeCode, Long id) throws PfxClientException {
        if (metaInstances.get(typeCode) == null || !metaInstances.get(typeCode).containsKey(id)) {
            try {
                String groupFieldName = typeCode.getIdentifierFieldNames()[0];
                Criterion criterion = PfxFilters.getSimpleCriteria(groupFieldName, id);
                TypeCode typeCodeMeta = typeCode.getMetaTypeCode();
                metaInstances.put(typeCode, groupByInstanceSorted(groupFieldName, GeneralApi.fetch(partition, typeCodeMeta.getShortName(), criterion)));
            } catch (Exception e) {
                metaInstances.put(typeCode, new HashMap<>());
                throw e;
            }
        }

        return metaInstances.get(typeCode).get(id);
    }
}
